@@ -5,6 +5,7 @@ Health.prototype.Schema =
 	"<a:example>" +
 		"<Max>100</Max>" +
 		"<RegenRate>1.0</RegenRate>" +
+		"<IdleRegenRate>0</IdleRegenRate>" +
 		"<DeathType>corpse</DeathType>" +
 	"</a:example>" +
 	"<element name='Max' a:help='Maximum hitpoints'>" +
@@ -16,13 +17,24 @@ Health.prototype.Schema =
 		"</element>" +
 	"</optional>" +
 	"<optional>" +
-		"<element name='SpawnEntityOnDeath' a:help='Entity template to spawn when this entity dies. Note: this is different than the corpse, which retains the original entity&apos;s appearance'>" +
-			"<text/>" +
+		"<element name='DamageVariants'>" +
+			"<oneOrMore>" +
+				"<element a:help='Name of the variant to select when health drops under the defined ratio'>" +
+					"<anyName/>" +
+					"<data type='decimal'>" +
+						"<param name='minInclusive'>0</param>" +
+						"<param name='maxInclusive'>1</param>" +
+					"</data>" +
+				"</element>" +
+			"</oneOrMore>" +
 		"</element>" +
 	"</optional>" +
 	"<element name='RegenRate' a:help='Hitpoint regeneration rate per second.'>" +
 		"<data type='decimal'/>" +
 	"</element>" +
+	"<element name='IdleRegenRate' a:help='Hitpoint regeneration rate per second when idle or garrisoned.'>" +
+		"<data type='decimal'/>" +
+	"</element>" +
 	"<element name='DeathType' a:help='Behaviour when the unit dies'>" +
 		"<choice>" +
 			"<value a:help='Disappear instantly'>vanish</value>" +
@@ -30,6 +42,11 @@ Health.prototype.Schema =
 			"<value a:help='Remain in the world with 0 health'>remain</value>" +
 		"</choice>" +
 	"</element>" +
+	"<optional>" +
+		"<element name='SpawnEntityOnDeath' a:help='Entity template to spawn when this entity dies. Note: this is different than the corpse, which retains the original entity&apos;s appearance'>" +
+			"<text/>" +
+		"</element>" +
+	"</optional>" +
 	"<element name='Unhealable' a:help='Indicates that the entity can not be healed by healer units'>" +
 		"<data type='boolean'/>" +
 	"</element>";
@@ -42,7 +59,9 @@ Health.prototype.Init = function()
 	// (Allowing 0 initial HP would break our death detection code)
 	this.hitpoints = +(this.template.Initial || this.GetMaxHitpoints());
 	this.regenRate = ApplyValueModificationsToEntity("Health/RegenRate", +this.template.RegenRate, this.entity);
+	this.idleRegenRate = ApplyValueModificationsToEntity("Health/IdleRegenRate", +this.template.IdleRegenRate, this.entity);
 	this.CheckRegenTimer();
+	this.UpdateActor();
 };
 
 //// Interface functions ////
@@ -74,7 +93,7 @@ Health.prototype.SetHitpoints = function(value)
 
 	var old = this.hitpoints;
 	this.hitpoints = Math.max(1, Math.min(this.GetMaxHitpoints(), value));
-	
+
 	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 	if (cmpRangeManager)
 	{
@@ -84,7 +103,7 @@ Health.prototype.SetHitpoints = function(value)
 			cmpRangeManager.SetEntityFlag(this.entity, "injured", false);
 	}
 
-	Engine.PostMessage(this.entity, MT_HealthChanged, { "from": old, "to": this.hitpoints });
+	this.RegisterHealthChanged(old);
 };
 
 Health.prototype.IsRepairable = function()
@@ -99,6 +118,11 @@ Health.prototype.IsUnhealable = function()
 		|| this.GetHitpoints() >= this.GetMaxHitpoints());
 };
 
+Health.prototype.GetIdleRegenRate = function()
+{
+	return this.idleRegenRate;
+};
+
 Health.prototype.GetRegenRate = function()
 {
 	return this.regenRate;
@@ -106,10 +130,17 @@ Health.prototype.GetRegenRate = function()
 
 Health.prototype.ExecuteRegeneration = function()
 {
-	var regen = this.GetRegenRate();
+	let regen = this.GetRegenRate();
+	if (this.GetIdleRegenRate() != 0)
+	{
+		let cmpUnitAI = Engine.QueryInterface(this.entity, IID_UnitAI);
+		if (cmpUnitAI && (cmpUnitAI.IsIdle() || cmpUnitAI.IsGarrisoned() && !cmpUnitAI.IsTurret()))
+			regen += this.GetIdleRegenRate();
+	}
+
 	if (regen > 0)
 		this.Increase(regen);
-	else 
+	else
 		this.Reduce(-regen);
 };
 
@@ -119,9 +150,9 @@ Health.prototype.ExecuteRegeneration = function()
 Health.prototype.CheckRegenTimer = function()
 {
 	// check if we need a timer
-	if (this.GetRegenRate() == 0 ||
-		this.GetHitpoints() == this.GetMaxHitpoints() && this.GetRegenRate() > 0 ||
-		this.GetHitpoints() == 0)
+	if (this.GetRegenRate() == 0 && this.GetIdleRegenRate() == 0 ||
+	    this.GetHitpoints() == this.GetMaxHitpoints() && this.GetRegenRate() >= 0 && this.GetIdleRegenRate() >= 0 ||
+	    this.GetHitpoints() == 0)
 	{
 		// we don't need a timer, disable if one exists
 		if (this.regenTimer)
@@ -172,42 +203,48 @@ Health.prototype.Reduce = function(amount)
 		// might get called multiple times)
 		if (this.hitpoints)
 		{
+			this.hitpoints = 0;
+			this.RegisterHealthChanged(oldHitpoints);
 			state.killed = true;
 
+			let cmpDeathDamage = Engine.QueryInterface(this.entity, IID_DeathDamage);
+			if (cmpDeathDamage)
+				cmpDeathDamage.CauseDeathDamage();
+
 			PlaySound("death", this.entity);
 
-			// If SpawnEntityOnDeath is set, spawn the entity
-			if(this.template.SpawnEntityOnDeath)
+			if (this.template.SpawnEntityOnDeath)
 				this.CreateDeathSpawnedEntity();
 
-			if (this.template.DeathType == "corpse")
+			switch (this.template.DeathType)
 			{
+			case "corpse":
 				this.CreateCorpse();
-				Engine.DestroyEntity(this.entity);
-			}
-			else if (this.template.DeathType == "vanish")
+				break;
+
+			case "remain":
 			{
-				Engine.DestroyEntity(this.entity);
-			}
-			else if (this.template.DeathType == "remain")
-			{
-				var resource = this.CreateCorpse(true);
+				let resource = this.CreateCorpse(true);
 				if (resource != INVALID_ENTITY)
-					Engine.BroadcastMessage(MT_EntityRenamed, { entity: this.entity, newentity: resource });
-				Engine.DestroyEntity(this.entity);
+					Engine.PostMessage(this.entity, MT_EntityRenamed, { "entity": this.entity, "newentity": resource });
 			}
 
-			this.hitpoints = 0;
+			case "vanish":
+				break;
 
-			Engine.PostMessage(this.entity, MT_HealthChanged, { "from": oldHitpoints, "to": this.hitpoints });
+			default:
+				error("Invalid template.DeathType: " + this.template.DeathType);
+				break;
+			}
+
+			Engine.DestroyEntity(this.entity);
 		}
 
 	}
 	else
 	{
 		this.hitpoints -= amount;
-
-		Engine.PostMessage(this.entity, MT_HealthChanged, { "from": oldHitpoints, "to": this.hitpoints });
+		this.RegisterHealthChanged(oldHitpoints);
 	}
 	state.change = this.hitpoints - oldHitpoints;
 	return state;
@@ -229,7 +266,7 @@ Health.prototype.Increase = function(amount)
 
 	var old = this.hitpoints;
 	this.hitpoints = Math.min(this.hitpoints + amount, this.GetMaxHitpoints());
-	
+
 	if (this.hitpoints == this.GetMaxHitpoints())
 	{
 		var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
@@ -237,8 +274,8 @@ Health.prototype.Increase = function(amount)
 			cmpRangeManager.SetEntityFlag(this.entity, "injured", false);
 	}
 
-	Engine.PostMessage(this.entity, MT_HealthChanged, { "from": old, "to": this.hitpoints });
-	// We return the old and the actual hp
+	this.RegisterHealthChanged(old);
+
 	return { "old": old, "new": this.hitpoints};
 };
 
@@ -280,7 +317,7 @@ Health.prototype.CreateCorpse = function(leaveResources)
 	cmpCorpseVisual.SetActorSeed(cmpVisual.GetActorSeed());
 
 	// Make it fall over
-	cmpCorpseVisual.SelectAnimation("death", true, 1.0, "");
+	cmpCorpseVisual.SelectAnimation("death", true, 1.0);
 
 	return corpse;
 };
@@ -312,30 +349,66 @@ Health.prototype.CreateDeathSpawnedEntity = function()
 	return spawnedEntity;
 };
 
+Health.prototype.UpdateActor = function()
+{
+	if (!this.template.DamageVariants)
+		return;
+	let ratio = this.GetHitpoints() / this.GetMaxHitpoints();
+	let newDamageVariant = "alive";
+	if (ratio > 0)
+	{
+		let minTreshold = 1;
+		for (let key in this.template.DamageVariants)
+		{
+			let treshold = +this.template.DamageVariants[key];
+			if (treshold < ratio || treshold > minTreshold)
+				continue;
+			newDamageVariant = key;
+			minTreshold = treshold;
+		}
+	}
+	else
+		newDamageVariant = "death";
+
+	if (this.damageVariant && this.damageVariant == newDamageVariant)
+		return;
+
+	this.damageVariant = newDamageVariant;
+
+	let cmpVisual = Engine.QueryInterface(this.entity, IID_Visual);
+	if (cmpVisual)
+		cmpVisual.SetVariant("health", newDamageVariant);
+};
+
 Health.prototype.OnValueModification = function(msg)
 {
 	if (msg.component != "Health")
 		return;
 
-	var oldMaxHitpoints = this.GetMaxHitpoints();
-	var newMaxHitpoints = Math.round(ApplyValueModificationsToEntity("Health/Max", +this.template.Max, this.entity));
+	let oldMaxHitpoints = this.GetMaxHitpoints();
+	let newMaxHitpoints = ApplyValueModificationsToEntity("Health/Max", +this.template.Max, this.entity);
 	if (oldMaxHitpoints != newMaxHitpoints)
 	{
-		var newHitpoints = Math.round(this.GetHitpoints() * newMaxHitpoints/oldMaxHitpoints);
+		let newHitpoints = this.GetHitpoints() * newMaxHitpoints/oldMaxHitpoints;
 		this.maxHitpoints = newMaxHitpoints;
 		this.SetHitpoints(newHitpoints);
 	}
 
-	var oldRegenRate = this.regenRate;
+	let oldRegenRate = this.regenRate;
 	this.regenRate = ApplyValueModificationsToEntity("Health/RegenRate", +this.template.RegenRate, this.entity);
 
-	if (this.regenRate != oldRegenRate)
+	let oldIdleRegenRate = this.idleRegenRate;
+	this.idleRegenRate = ApplyValueModificationsToEntity("Health/IdleRegenRate", +this.template.IdleRegenRate, this.entity);
+
+	if (this.regenRate != oldRegenRate || this.idleRegenRate != oldIdleRegenRate)
 		this.CheckRegenTimer();
 };
 
-Health.prototype.OnHealthChanged = function()
+Health.prototype.RegisterHealthChanged = function(from)
 {
 	this.CheckRegenTimer();
+	this.UpdateActor();
+	Engine.PostMessage(this.entity, MT_HealthChanged, { "from": from, "to": this.hitpoints });
 };
 
 Engine.RegisterComponentType(IID_Health, "Health", Health);
