@@ -8,10 +8,15 @@ const g_MapSizes = prepareForDropdown(g_Settings && g_Settings.MapSizes);
  */
 const g_MapTypes = prepareForDropdown(g_Settings && g_Settings.MapTypes);
 
+/**
+ * Used for civ settings display of the selected game.
+ */
+const g_CivData = loadCivData(false, false);
+
 /**
  * A symbol which is prepended to the username of moderators.
  */
-const g_ModeratorPrefix = "@";
+var g_ModeratorPrefix = "@";
 
 /**
  * Current username. Cannot contain whitespace.
@@ -26,21 +31,22 @@ const g_LobbyServer = Engine.ConfigDB_GetValue("user", "lobby.server");
 /**
  * Current games will be listed in these colors.
  */
-const g_GameColors = {
+var g_GameColors = {
 	"init": "0 219 0",
 	"waiting": "255 127 0",
-	"running": "219 0 0"
+	"running": "219 0 0",
+	"incompatible": "gray"
 };
 
 /**
  * Initial sorting order of the gamelist.
  */
-const g_GameStatusOrder = ["init", "waiting", "running"];
+var g_GameStatusOrder = ["init", "waiting", "running", "incompatible"];
 
 /**
  * The playerlist will be assembled using these values.
  */
-const g_PlayerStatuses = {
+var g_PlayerStatuses = {
 	"available": { "color": "0 219 0",     "status": translate("Online") },
 	"away":      { "color": "229 76 13",   "status": translate("Away") },
 	"playing":   { "color": "200 0 0",     "status": translate("Busy") },
@@ -48,7 +54,7 @@ const g_PlayerStatuses = {
 	"unknown":   { "color": "178 178 178", "status": translateWithContext("lobby presence", "Unknown") }
 };
 
-const g_RoleNames = {
+var g_RoleNames = {
 	"moderator": translate("Moderator"),
 	"participant": translate("Player"),
 	"visitor": translate("Muted Player")
@@ -57,22 +63,27 @@ const g_RoleNames = {
 /**
  * Color for error messages in the chat.
  */
-const g_SystemColor = "150 0 0";
+var g_SystemColor = "150 0 0";
 
 /**
  * Color for private messages in the chat.
  */
-const g_PrivateMessageColor = "0 150 0";
+var g_PrivateMessageColor = "0 150 0";
 
 /**
  * Used for highlighting the sender of chat messages.
  */
-const g_SenderFont = "sans-bold-13";
+var g_SenderFont = "sans-bold-13";
 
 /**
  * Color to highlight chat commands in the explanation.
  */
-const g_ChatCommandColor = "200 200 255";
+var g_ChatCommandColor = "200 200 255";
+
+/**
+ * Indicates if the lobby is opened as a dialog or window.
+ */
+var g_Dialog = false;
 
 /**
  * All chat messages received since init (i.e. after lobby join and after returning from a game).
@@ -88,7 +99,7 @@ var g_UserRating = "";
 /**
  * All games currently running.
  */
-var g_GameList = {};
+var g_GameList = [];
 
 /**
  * Used to restore the selection after updating the playerlist.
@@ -111,33 +122,43 @@ var g_SelectedGamePort = "";
 var g_Kicked = false;
 
 /**
- * Notifications sent by XmppClient.cpp
+ * Whether the player was already asked to reconnect to the lobby.
+ * Ensures that no more than one message box is opened at a time.
+ */
+var g_AskedReconnect = false;
+
+/**
+ * Processing of notifications sent by XmppClient.cpp.
+ *
+ * @returns true if the playerlist GUI must be updated.
  */
 var g_NetMessageTypes = {
 	"system": {
 		// Three cases are handled in prelobby.js
-		"registered": msg => {
-		},
+		"registered": msg => false,
 		"connected": msg => {
+
+			g_AskedReconnect = false;
+			updateConnectedState();
+			return false;
 		},
 		"disconnected": msg => {
 
 			updateGameList();
 			updateLeaderboard();
-			updatePlayerList();
-
-			Engine.GetGUIObjectByName("chatInput").hidden = true;
-
-			for (let button of ["host", "leaderboard", "userprofile", "toggleBuddy"])
-				Engine.GetGUIObjectByName(button + "Button").enabled = false;
-			Engine.GetGUIObjectByName("chatInput").hidden = true;
+			updateConnectedState();
 
 			if (!g_Kicked)
+			{
 				addChatMessage({
 					"from": "system",
 					"time": msg.time,
-					"text": translate("Disconnected.") + " " + msg.text
+					"text": translate("Disconnected.") + " " + msg.reason
 				});
+				reconnectMessageBox();
+			}
+
+			return true;
 		},
 		"error": msg => {
 			addChatMessage({
@@ -145,50 +166,64 @@ var g_NetMessageTypes = {
 				"time": msg.time,
 				"text": msg.text
 			});
+			return false;
 		}
 	},
 	"chat": {
 		"subject": msg => {
-			updateSubject(msg.text);
+			updateSubject(msg.subject);
+
+			if (msg.nick)
+				addChatMessage({
+					"text": "/special " + sprintf(translate("%(nick)s changed the lobby subject to %(subject)s"), {
+						"nick": msg.nick,
+						"subject": msg.subject
+					}),
+					"time": msg.time,
+					"isSpecial": true
+				});
+			return false;
 		},
 		"join": msg => {
 			addChatMessage({
 				"text": "/special " + sprintf(translate("%(nick)s has joined."), {
-					"nick": msg.text
+					"nick": msg.nick
 				}),
 				"time": msg.time,
 				"isSpecial": true
 			});
+			return true;
 		},
 		"leave": msg => {
 			addChatMessage({
 				"text": "/special " + sprintf(translate("%(nick)s has left."), {
-					"nick": msg.text
+					"nick": msg.nick
 				}),
 				"time": msg.time,
 				"isSpecial": true
 			});
 
-			if (msg.text == g_Username)
+			if (msg.nick == g_Username)
 				Engine.DisconnectXmppClient();
+
+			return true;
 		},
-		"presence": msg => {
-		},
+		"presence": msg => true,
 		"role": msg => {
 			Engine.GetGUIObjectByName("chatInput").hidden = Engine.LobbyGetPlayerRole(g_Username) == "visitor";
 
-			let me = g_Username == msg.text;
-			let role = Engine.LobbyGetPlayerRole(msg.text);
+			let me = g_Username == msg.nick;
+			let newrole = Engine.LobbyGetPlayerRole(msg.nick);
 			let txt =
-				role == "visitor" ?
+				newrole == "visitor" ?
 					me ?
 						translate("You have been muted.") :
 						translate("%(nick)s has been muted.") :
-				role == "moderator" ?
+				newrole == "moderator" ?
 					me ?
 						translate("You are now a moderator.") :
 						translate("%(nick)s is now a moderator.") :
-				msg.data == "visitor" ?
+				msg.oldrole == "visitor" ?
 					me ?
 						translate("You have been unmuted.") :
 						translate("%(nick)s has been unmuted.") :
@@ -197,53 +232,82 @@ var g_NetMessageTypes = {
 						translate("%(nick)s is not a moderator anymore.");
 
 			addChatMessage({
-				"text": "/special " + sprintf(txt, { "nick": msg.text }),
+				"text": "/special " + sprintf(txt, { "nick": msg.nick }),
 				"time": msg.time,
 				"isSpecial": true
 			});
 
-			if (g_SelectedPlayer == msg.text)
+			if (g_SelectedPlayer == msg.nick)
 				updateUserRoleText(g_SelectedPlayer);
+
+			return false;
 		},
 		"nick": msg => {
 			addChatMessage({
 				"text": "/special " + sprintf(translate("%(oldnick)s is now known as %(newnick)s."), {
-					"oldnick": msg.text,
-					"newnick": msg.data
+					"oldnick": msg.oldnick,
+					"newnick": msg.newnick
 				}),
 				"time": msg.time,
 				"isSpecial": true
 			});
+			return true;
 		},
 		"kicked": msg => {
-			handleKick(false, msg.text, msg.data || "", msg.time);
+			handleKick(false, msg.nick, msg.reason, msg.time, msg.historic);
+			return true;
 		},
 		"banned": msg => {
-			handleKick(true, msg.text, msg.data || "", msg.time);
+			handleKick(true, msg.nick, msg.reason, msg.time, msg.historic);
+			return true;
 		},
 		"room-message": msg => {
 			addChatMessage({
 				"from": escapeText(msg.from),
 				"text": escapeText(msg.text),
-				"time": msg.time
+				"time": msg.time,
+				"historic": msg.historic
 			});
+			return false;
 		},
 		"private-message": msg => {
-			if (Engine.LobbyGetPlayerRole(msg.from) == "moderator")
+			// Announcements and the Message of the Day are sent by the server directly
+			if (!msg.from)
+				messageBox(
+					400, 250,
+					msg.text.trim(),
+					translate("Notice")
+				);
+
+			// We intend to not support private messages between users
+			if (!msg.from || Engine.LobbyGetPlayerRole(msg.from) == "moderator")
 				// some XMPP clients send trailing whitespace
 				addChatMessage({
-					"from": escapeText(msg.from),
+					"from": escapeText(msg.from || "system"),
 					"text": escapeText(msg.text.trim()),
 					"time": msg.time,
-					"private" : true
+					"historic": msg.historic,
+					"private": true
 				});
+			return false;
 		}
 	},
 	"game": {
-		"gamelist": msg => updateGameList(),
-		"profile": msg => updateProfile(),
-		"leaderboard": msg => updateLeaderboard(),
-		"ratinglist": msg => updatePlayerList()
+		"gamelist": msg => {
+			updateGameList();
+			return false;
+		},
+		"profile": msg => {
+			updateProfile();
+			return false;
+		},
+		"leaderboard": msg => {
+			updateLeaderboard();
+			return false;
+		},
+		"ratinglist": msg => {
+			return true;
+		}
 	}
 };
 
@@ -291,7 +355,7 @@ var g_ChatCommands = {
 				if (!g_ChatCommands[command].moderatorOnly || isModerator)
 					// Translation: Chat command help format
 					text += "\n" + sprintf(translate("%(command)s - %(description)s"), {
-						"command": '[color="' + g_ChatCommandColor + '"]' + command + '[/color]',
+						"command": coloredText(command, g_ChatCommandColor),
 						"description": g_ChatCommands[command].description
 					});
 
@@ -320,7 +384,7 @@ var g_ChatCommands = {
 	"quit": {
 		"description": translate("Return to the main menu."),
 		"handler": args => {
-			returnToMainMenu();
+			leaveLobby();
 			return false;
 		}
 	}
@@ -333,16 +397,20 @@ var g_ChatCommands = {
  */
 function init(attribs)
 {
+	g_Dialog = attribs && attribs.dialog;
+
 	if (!g_Settings)
 	{
-		returnToMainMenu();
+		leaveLobby();
 		return;
 	}
 
 	initMusic();
 	global.music.setState(global.music.states.MENU);
 
+	initDialogStyle();
 	initGameFilters();
+	updateConnectedState();
 
 	Engine.LobbySetPlayerPresence("available");
 
@@ -354,6 +422,69 @@ function init(attribs)
 
 	updateToggleBuddy();
 	Engine.GetGUIObjectByName("chatInput").tooltip = colorizeAutocompleteHotkey();
+
+	// Get all messages since the login
+	for (let msg of Engine.LobbyGuiPollHistoricMessages())
+		g_NetMessageTypes[msg.type][msg.level](msg);
+
+	if (!Engine.IsXmppClientConnected())
+		reconnectMessageBox();
+}
+
+function reconnectMessageBox()
+{
+	if (g_AskedReconnect)
+		return;
+
+	g_AskedReconnect = true;
+
+	messageBox(
+		400, 200,
+		translate("You have been disconnected from the lobby. Do you want to reconnect?"),
+		translate("Confirmation"),
+		[translate("No"), translate("Yes")],
+		[null, Engine.ConnectXmppClient]);
+}
+
+/**
+ * Set style of GUI elements and the window style.
+ */
+function initDialogStyle()
+{
+	let lobbyWindow = Engine.GetGUIObjectByName("lobbyWindow");
+	lobbyWindow.sprite = g_Dialog ? "ModernDialog" : "ModernWindow";
+	lobbyWindow.size = g_Dialog ? "42 42 100%-42 100%-42" : "0 0 100% 100%";
+	Engine.GetGUIObjectByName("lobbyWindowTitle").size = g_Dialog ? "50%-128 -16 50%+128 16" : "50%-128 4 50%+128 36";
+
+	Engine.GetGUIObjectByName("leaveButton").caption = g_Dialog ?
+		translateWithContext("previous page", "Back") :
+		translateWithContext("previous page", "Main Menu");
+
+	Engine.GetGUIObjectByName("hostButton").hidden = g_Dialog;
+	Engine.GetGUIObjectByName("joinGameButton").hidden = g_Dialog;
+	Engine.GetGUIObjectByName("gameInfoEmpty").size = "0 0 100% 100%-24" + (g_Dialog ? "" : "-30");
+	Engine.GetGUIObjectByName("gameInfo").size = "0 0 100% 100%-24" + (g_Dialog ? "" : "-60");
+
+	Engine.GetGUIObjectByName("middlePanel").size = "20%+5 " + (g_Dialog ? "18" : "40") + " 100%-255 100%-20";
+	Engine.GetGUIObjectByName("rightPanel").size = "100%-250 " + (g_Dialog ? "18" : "40") + " 100%-20 100%-20";
+	Engine.GetGUIObjectByName("leftPanel").size = "20 " + (g_Dialog ? "18" : "40") + " 20% 100%-315";
+
+	if (g_Dialog)
+	{
+		Engine.GetGUIObjectByName("lobbyDialogToggle").onPress = leaveLobby;
+		Engine.GetGUIObjectByName("cancelDialog").onPress = leaveLobby;
+	}
+}
+
+/**
+ * Set style of GUI elements according to the connection state of the lobby.
+ */
+function updateConnectedState()
+{
+	Engine.GetGUIObjectByName("chatInput").hidden = !Engine.IsXmppClientConnected();
+
+	for (let button of ["host", "leaderboard", "userprofile", "toggleBuddy"])
+		Engine.GetGUIObjectByName(button + "Button").enabled = Engine.IsXmppClientConnected();
 }
 
 function updateLobbyColumns()
@@ -374,15 +505,23 @@ function updateLobbyColumns()
 	// Keep filters right above the according column
 	let playersNumberFilter = Engine.GetGUIObjectByName("playersNumberFilter");
 	let size = playersNumberFilter.size;
-	size.rleft = gameRating ? 74: 90;
-	size.rright = gameRating ? 84: 100;
+	size.rleft = gameRating ? 74 : 90;
+	size.rright = gameRating ? 84 : 100;
 	playersNumberFilter.size = size;
 }
 
-function returnToMainMenu()
+function leaveLobby()
 {
-	Engine.StopXmppClient();
-	Engine.SwitchGuiPage("page_pregame.xml");
+	if (g_Dialog)
+	{
+		Engine.LobbySetPlayerPresence("playing");
+		Engine.PopGuiPage();
+	}
+	else
+	{
+		Engine.StopXmppClient();
+		Engine.SwitchGuiPage("page_pregame.xml");
+	}
 }
 
 function initGameFilters()
@@ -400,7 +539,7 @@ function initGameFilters()
 	mapTypeFilter.list = [translateWithContext("map", "Any")].concat(g_MapTypes.Title);
 	mapTypeFilter.list_data = [""].concat(g_MapTypes.Name);
 
-	let gameRatingOptions = ["<1000", "<1100","<1200",">1200",">1300",">1400",">1500"].reverse();
+	let gameRatingOptions = [">1500", ">1400", ">1300", ">1200", "<1200", "<1100", "<1000"];
 	gameRatingOptions = prepareForDropdown(gameRatingOptions.map(r => ({
 		"value": r,
 		"label": sprintf(
@@ -475,7 +614,7 @@ function filterGame(game)
 	return false;
 }
 
-function handleKick(banned, nick, reason, time)
+function handleKick(banned, nick, reason, time, historic)
 {
 	let kickString = nick == g_Username ?
 		banned ?
@@ -495,6 +634,7 @@ function handleKick(banned, nick, reason, time)
 		addChatMessage({
 			"text": "/special " + sprintf(kickString, { "nick": nick }) + " " + reason,
 			"time": time,
+			"historic": historic,
 			"isSpecial": true
 		});
 		return;
@@ -519,8 +659,6 @@ function handleKick(banned, nick, reason, time)
 
 /**
  * Update the subject GUI object.
- *
- * @param {string} newSubject
  */
 function updateSubject(newSubject)
 {
@@ -611,14 +749,10 @@ function updatePlayerList()
 			warn("Unknown presence:" + player.presence);
 
 		let statusColor = g_PlayerStatuses[presence].color;
-		let coloredName = colorPlayerName((player.role == "moderator" ? g_ModeratorPrefix : "") + player.name);
-		let coloredPresence = '[color="' + statusColor + '"]' + g_PlayerStatuses[presence].status + "[/color]";
-		let coloredRating = '[color="' + statusColor + '"]' + rating + "[/color]";
-
-		buddyStatusList.push(player.isBuddy ? '[color="' + statusColor + '"]' + g_BuddySymbol + '[/color]' : "");
-		playerList.push(coloredName);
-		presenceList.push(coloredPresence);
-		ratingList.push(coloredRating);
+		buddyStatusList.push(player.isBuddy ? coloredText(g_BuddySymbol, statusColor) : "");
+		playerList.push(colorPlayerName((player.role == "moderator" ? g_ModeratorPrefix : "") + player.name));
+		presenceList.push(coloredText(g_PlayerStatuses[presence].status, statusColor));
+		ratingList.push(coloredText(rating, statusColor));
 		nickList.push(player.name);
 	}
 
@@ -650,10 +784,7 @@ function toggleBuddy()
 
 	updateToggleBuddy();
 
-	// Don't save empty strings to the config file
-	let buddies = g_Buddies.filter(nick => nick).join(g_BuddyListDelimiter) || g_BuddyListDelimiter;
-	Engine.ConfigDB_CreateValue("user", "lobby.buddies", buddies);
-	Engine.ConfigDB_WriteValueToFile("user", "lobby.buddies", buddies, "config/user.cfg");
+	saveSettingAndWriteToUserConfig("lobby.buddies", g_Buddies.filter(nick => nick).join(g_BuddyListDelimiter) || g_BuddyListDelimiter);
 
 	updatePlayerList();
 	updateGameList();
@@ -674,10 +805,10 @@ function selectGameFromPlayername()
 	for (let i = 0; i < g_GameList.length; ++i)
 		for (let player of stringifiedTeamListToPlayerData(g_GameList[i].players))
 		{
-			let nick = splitRatingFromNick(player.Name)[0];
-			if (g_SelectedPlayer != nick)
+			if (g_SelectedPlayer != splitRatingFromNick(player.Name).nick)
 				continue;
 
+			gameList.auto_scroll = true;
 			if (player.Team == "observer")
 			{
 				foundAsObserver = true;
@@ -834,7 +965,7 @@ function updateLeaderboard()
 	{
 		list_name.push(boardList[i].name);
 		list_rating.push(boardList[i].rating);
-		list_rank.push(+i+1);
+		list_rank.push(+i + 1);
 		list.push(boardList[i].name);
 	}
 
@@ -871,13 +1002,13 @@ function updateGameList()
 
 		for (let player of stringifiedTeamListToPlayerData(game.players))
 		{
-			let [nick, rating] = splitRatingFromNick(player.Name);
+			let playerNickRating = splitRatingFromNick(player.Name);
 
 			if (player.Team != "observer")
-				playerRatings.push(rating || g_DefaultLobbyRating);
+				playerRatings.push(playerNickRating.rating || g_DefaultLobbyRating);
 
 			// Sort games with playing buddies above games with spectating buddies
-			if (game.hasBuddies < 2 && g_Buddies.indexOf(nick) != -1)
+			if (game.hasBuddies < 2 && g_Buddies.indexOf(playerNickRating.nick) != -1)
 				game.hasBuddies = player.Team == "observer" ? 1 : 2;
 		}
 
@@ -886,6 +1017,18 @@ function updateGameList()
 				Math.round(playerRatings.reduce((sum, current) => sum + current) / playerRatings.length) :
 				g_DefaultLobbyRating;
 
+		try
+		{
+			game.mods = JSON.parse(game.mods);
+		}
+		catch (e)
+		{
+			game.mods = [];
+		}
+
+		if (!hasSameMods(game.mods, Engine.GetEngineInfo().mods))
+			game.state = "incompatible";
+
 		return game;
 	}).filter(game => !filterGame(game)).sort((a, b) => {
 		let sortA, sortB;
@@ -939,8 +1082,8 @@ function updateGameList()
 		if (game.ip == g_SelectedGameIP && game.port == g_SelectedGamePort)
 			selectedGameIndex = +i;
 
-		list_buddy.push(game.hasBuddies ? '[color="' + g_GameColors[game.state] + '"]' + g_BuddySymbol + '[/color]' : "");
-		list_name.push('[color="' + g_GameColors[game.state] + '"]' + gameName);
+		list_buddy.push(game.hasBuddies ? coloredText(g_BuddySymbol, g_GameColors[game.state]) : "");
+		list_name.push(coloredText(gameName, g_GameColors[game.state]));
 		list_mapName.push(translateMapTitle(game.niceMapName));
 		list_mapSize.push(translateMapSize(game.mapSize));
 		list_mapType.push(g_MapTypes.Title[mapTypeIdx] || "");
@@ -961,6 +1104,8 @@ function updateGameList()
 	// Change these last, otherwise crash
 	gamesBox.list = list;
 	gamesBox.list_data = list_data;
+
+	gamesBox.auto_scroll = false;
 	gamesBox.selected = selectedGameIndex;
 
 	updateGameSelection();
@@ -974,7 +1119,7 @@ function updateGameSelection()
 	let game = selectedGame();
 
 	Engine.GetGUIObjectByName("gameInfo").hidden = !game;
-	Engine.GetGUIObjectByName("joinGameButton").hidden = !game;
+	Engine.GetGUIObjectByName("joinGameButton").hidden = g_Dialog || !game;
 	Engine.GetGUIObjectByName("gameInfoEmpty").hidden = game;
 
 	if (!game)
@@ -996,7 +1141,7 @@ function updateGameSelection()
 		sgGameStartTime.caption = sprintf(
 			// Translation: %(time)s is the hour and minute here.
 			translate("Game started at %(time)s"), {
-				"time": Engine.FormatMillisecondsIntoDateStringLocal(+game.startTime*1000, translate("HH:mm"))
+				"time": Engine.FormatMillisecondsIntoDateStringLocal(+game.startTime * 1000, translate("HH:mm"))
 			});
 
 	sgNbPlayers.caption = sprintf(
@@ -1032,13 +1177,31 @@ function selectedGame()
 function joinButton()
 {
 	let game = selectedGame();
-	if (!game)
+	if (!game || g_Dialog)
 		return;
 
 	let rating = getRejoinRating(game);
 	let username = rating ? g_Username + " (" + rating + ")" : g_Username;
 
-	if (game.state == "init" || stringifiedTeamListToPlayerData(game.players).some(player => player.Name == username))
+	if (game.state == "incompatible")
+		messageBox(
+			400, 200,
+			translate("Your active mods do not match the mods of this game.") + "\n\n" +
+				comparedModsString(game.mods, Engine.GetEngineInfo().mods) + "\n\n" +
+				translate("Do you want to switch to the mod selection page?"),
+			translate("Incompatible mods"),
+			[translate("No"), translate("Yes")],
+			[
+				null,
+				() => {
+					Engine.StopXmppClient();
+					Engine.SwitchGuiPage("page_modmod.xml", {
+						"cancelbutton": true
+					});
+				}
+			]
+		);
+	else if (game.state == "init" || stringifiedTeamListToPlayerData(game.players).some(player => player.Name == username))
 		joinSelectedGame();
 	else
 		messageBox(
@@ -1102,9 +1265,9 @@ function getRejoinRating(game)
 {
 	for (let player of stringifiedTeamListToPlayerData(game.players))
 	{
-		let [nick, rating] = splitRatingFromNick(player.Name);
-		if (nick == g_Username)
-			return rating;
+		let playerNickRating = splitRatingFromNick(player.Name);
+		if (playerNickRating.nick == g_Username)
+			return playerNickRating.rating;
 	}
 	return g_UserRating;
 }
@@ -1128,9 +1291,11 @@ function onTick()
 {
 	updateTimers();
 
+	let updateList = false;
+
 	while (true)
 	{
-		let msg = Engine.LobbyGuiPollMessage();
+		let msg = Engine.LobbyGuiPollNewMessage();
 		if (!msg)
 			break;
 
@@ -1144,13 +1309,15 @@ function onTick()
 			warn("Unrecognised message level: " + msg.level);
 			continue;
 		}
-		g_NetMessageTypes[msg.type][msg.level](msg);
 
-		// To improve performance, only update the playerlist GUI when
-		// the last update in the current stack is processed
-		if (msg.type == "chat" && Engine.LobbyGetMucMessageCount() == 0)
-			updatePlayerList();
+		if (g_NetMessageTypes[msg.type][msg.level](msg))
+			updateList = true;
 	}
+
+	// To improve performance, only update the playerlist GUI when
+	// the last update in the current stack is processed
+	if (updateList)
+		updatePlayerList();
 }
 
 /**
@@ -1190,7 +1357,7 @@ function handleChatCommand(text)
 			"from": "system",
 			"text": sprintf(
 				translate("The command '%(cmd)s' is not supported."), {
-					"cmd": '[color="' + g_ChatCommandColor + '"]' + cmd + '[/color]'
+					"cmd": coloredText(cmd, g_ChatCommandColor)
 				})
 		});
 		return false;
@@ -1202,7 +1369,7 @@ function handleChatCommand(text)
 			"from": "system",
 			"text": sprintf(
 				translate("The command '%(cmd)s' is restricted to moderators."), {
-					"cmd": '[color="' + g_ChatCommandColor + '"]' + cmd + '[/color]'
+					"cmd": coloredText(cmd, g_ChatCommandColor)
 				})
 		});
 		return false;
@@ -1227,7 +1394,9 @@ function addChatMessage(msg)
 		if (g_Username != msg.from)
 		{
 			msg.text = msg.text.replace(g_Username, colorPlayerName(g_Username));
-			notifyUser(g_Username, msg.text);
+
+			if (!msg.historic && msg.text.toLowerCase().indexOf(g_Username.toLowerCase()) != -1)
+				soundNotification("nick");
 		}
 	}
 
@@ -1247,7 +1416,7 @@ function ircSplit(string)
 	let idx = string.indexOf(' ');
 
 	if (idx != -1)
-		return [string.substr(1,idx-1), string.substr(idx+1)];
+		return [string.substr(1, idx - 1), string.substr(idx + 1)];
 
 	return [string.substr(1), ""];
 }
@@ -1264,7 +1433,7 @@ function ircFormat(msg)
 	let coloredFrom = msg.from && colorPlayerName(msg.from);
 
 	// Handle commands allowed past handleChatCommand.
-	if (msg.text[0] == '/')
+	if (msg.text && msg.text[0] == '/')
 	{
 		let [command, message] = ircSplit(msg.text);
 		switch (command)
@@ -1319,6 +1488,8 @@ function ircFormat(msg)
 			}
 			break;
 		}
+		default:
+			return "";
 		}
 	}
 	else
@@ -1328,8 +1499,7 @@ function ircFormat(msg)
 		// Translation: IRC message prefix.
 		if (msg.private)
 			senderString = sprintf(translateWithContext("lobby private message", "(%(private)s) <%(sender)s>"), {
-				"private": '[color="' + g_PrivateMessageColor + '"]' +
-							translate("Private") + '[/color]',
+				"private": coloredText(translate("Private"), g_PrivateMessageColor),
 				"sender": coloredFrom
 			});
 		else
@@ -1360,14 +1530,14 @@ function ircFormat(msg)
 
 	// Translation: IRC message format when there is a time prefix.
 	return sprintf(translate("%(time)s %(message)s"), {
-		"time": senderFont(timePrefixString),
+		"time": timePrefixString,
 		"message": formattedMessage
 	});
- }
+}
 
 /**
  * Generate a (mostly) unique color for this player based on their name.
- * @see http://stackoverflow.com/questions/3426404/create-a-hexadecimal-colour-based-on-a-string-with-jquery-javascript
+ * @see https://stackoverflow.com/questions/3426404/create-a-hexadecimal-colour-based-on-a-string-with-jquery-javascript
  * @param {string} playername
  */
 function getPlayerColor(playername)
@@ -1396,13 +1566,14 @@ function getPlayerColor(playername)
  */
 function colorPlayerName(playername, rating)
 {
-	return '[color="' + getPlayerColor(playername.replace(g_ModeratorPrefix, "")) + '"]' +
+	return coloredText(
 		(rating ? sprintf(
 			translate("%(nick)s (%(rating)s)"), {
 				"nick": playername,
 				"rating": rating
-			}) :
-		playername) + '[/color]';
+			}) : playername
+		),
+		getPlayerColor(playername.replace(g_ModeratorPrefix, "")));
 }
 
 function senderFont(text)
